﻿/*
  Antenna House PDF Tool API V6.0
  .Net Interface sample program

  概要：レイヤーの描画　指定したページにもう一つのPDFを重ねて表示します。
        レイヤーのZオーダーを設定できます。
  Copyright (C) 2018- Antenna House, Inc. All rights reserved.
  Antenna House is a trademark of [Antenna House, Inc.]

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/

using PdfTkNet;
using System;

namespace cookbook
{
    public class DrawLayerSetZorder
    {
        // そのクラスのusageを表示する関数
        private static void printUsage()
        {
            Console.WriteLine("usage: DrawLayerSetZorder in-pdf-file out-pdf-file" +
                               " page-num insert-pdf-file page-num-to-insert set-z-order");
            Console.WriteLine("set-z-order:");
            Console.WriteLine("0:レイヤーのZオーダーを背面する" +
                               " 1:レイヤーのZオーダーを前面にする");
        }

        /**
         * @param args the command line arguments
         */
        public static void Main(String[] args)
        {
            if (args.Length < 6)
            {
                printUsage();
                return;
            }

            // コマンドライン引数の判定
            bool setZorder = true;
            try
            {
                setZorder =
                    readBoolArgs(args[5], "set-z-order は 0か1で指定してください。");
            }
            catch (ArgumentException ex)
            {
                Console.WriteLine(ex.Message);
                printUsage(); // usageメッセージの表示
                return;
            }


            try
            {
                using (PtlParamInput inputFile = new PtlParamInput(args[0]))
                using (PtlParamOutput outputFile = new PtlParamOutput(args[1]))
                using (PtlPDFDocument doc = new PtlPDFDocument())
                {
                    // PDFファイルをロードします。
                    doc.load(inputFile);
                    // コマンドライン引数の判定
                    int pageToAdd = int.Parse(args[2]);
                    int numPages = doc.getPageCount();
                    Console.WriteLine("ページ数：" + numPages);
                    if ((numPages < 0) || (numPages < pageToAdd))
                    {
                        Console.WriteLine("page-numは入力PDFの全ページ数よりも小さい正の値を" +
                                           "指定してください。");
                        return;
                    }
                    String insertPDFPath = args[3];
                    int insertPageNum = int.Parse(args[4]);

                    using (PtlPages pages = doc.getPages())
                    { // ページコンテナの取得
                      // ページコンテナが空かどうか
                        if (pages.isEmpty())
                        {
                            Console.WriteLine("ページコンテナが空");
                            return;
                        }

                        // ページの取得(パラメータindexは0が先頭のため1を引く)
                        using (PtlPage page = pages.get(pageToAdd - 1))
                        {
                            //レイヤーの描画
                            drawLayerSetZOrder(page, insertPDFPath, insertPageNum, setZorder);
                        }
                    }

                    // ファイルに保存します。
                    doc.save(outputFile);
                }
            }
            catch (PtlException pex)
            {
                Console.WriteLine("PtlException : ErrorCode = " + pex.getErrorCode() +
                                   "\n  " + pex.getErrorMessage());
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                Console.WriteLine(ex.StackTrace);
            }
            finally
            {
                Console.WriteLine("-- 完了 --");
            }
        }

        public static void drawLayerSetZOrder(PtlPage page, String insertPDFPath,
                                            int insertPageNum, bool setZorder)
        {
            using (PtlSize pageSize = page.getSize())    // ページサイズ
            using (PtlContent content = page.getContent()) // ページコンテントの取得
            using (PtlPDFDocument docToInsert = new PtlPDFDocument())
            //挿入するPDF
            using (PtlParamInput inputFileToInsert = new PtlParamInput(insertPDFPath))
            // 描画矩形の指定（PDFいっぱいに指定）
            using (PtlRect rect = new PtlRect(0.0f,
                                       0.0f,
                                       pageSize.getWidth(),
                                       pageSize.getHeight()))
            {
                // PDFファイルをロードします。
                docToInsert.load(inputFileToInsert);

                using (PtlPages pagesToInsert = docToInsert.getPages())
                { // ページコンテナの取得
                  // ページコンテナが空かどうか
                    if (pagesToInsert.isEmpty())
                    {
                        Console.WriteLine("ページコンテナが空");
                        return;
                    }

                    // ページの取得(パラメータindexは0が先頭のため1を引く)
                    // レイヤーのパラメーターの取得
                    using (PtlPage pageInsert = pagesToInsert.get(insertPageNum - 1))
                    using (PtlParamDrawLayer param = new PtlParamDrawLayer())
                    {
                        // レイヤーの配置のZオーダーを設定
                        if (setZorder)
                        {
                            param.setZorder(PtlParamDrawLayer.ZORDER.ZORDER_FRONT);
                        }
                        else
                        {
                            param.setZorder(PtlParamDrawLayer.ZORDER.ZORDER_BACK);
                        }

                        // レイヤーにするページ
                        param.setPage(pageInsert);
                        // レイヤーの名前
                        param.setName("Layer2");
                        // レイヤーの表示/非表示を指定
                        param.setShow(PtlParamDrawLayer.SHOW.SHOW_ON);
                        // レイヤーの描画
                        content.drawLayer(rect, PtlContent.ALIGN.ALIGN_CENTER, param);
                    }
                }
            }
        }

        /**
         * 0または1を入力されたargsにより、trueまたはfalseを返すメソッド。
         *
         * @param args 与えられるコマンドライン引数。0または1でtrueまたはfalseを指定する。
         * @param errorMessage argsが0か1でなかった場合に出力されるエラーメッセージを指定する。
         * @return argsの数値を読み取った結果を戻す
         * @throws ArgumentException argsが0か1でなかった場合に発生。
         */
        public static bool readBoolArgs(String args, String errorMessage)
        {
            bool trueOrFalse = false;

            // argsの読み取り
            switch (args)
            {
                case "0":
                    trueOrFalse = false;
                    break;
                case "1":
                    trueOrFalse = true;
                    break;
                default:
                    throw new ArgumentException(errorMessage);
            }

            return trueOrFalse;
        }
    }
}
